/**
 * Copyright (C) 2011 Angelo Zerr <angelo.zerr@gmail.com> and Pascal Leclercq <pascal.leclercq@gmail.com>
 *
 * All rights reserved.
 *
 * Permission is hereby granted, free  of charge, to any person obtaining
 * a  copy  of this  software  and  associated  documentation files  (the
 * "Software"), to  deal in  the Software without  restriction, including
 * without limitation  the rights to  use, copy, modify,  merge, publish,
 * distribute,  sublicense, and/or sell  copies of  the Software,  and to
 * permit persons to whom the Software  is furnished to do so, subject to
 * the following conditions:
 *
 * The  above  copyright  notice  and  this permission  notice  shall  be
 * included in all copies or substantial portions of the Software.
 *
 * THE  SOFTWARE IS  PROVIDED  "AS  IS", WITHOUT  WARRANTY  OF ANY  KIND,
 * EXPRESS OR  IMPLIED, INCLUDING  BUT NOT LIMITED  TO THE  WARRANTIES OF
 * MERCHANTABILITY,    FITNESS    FOR    A   PARTICULAR    PURPOSE    AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE,  ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package fr.opensagres.xdocreport.document.debug;

import java.io.IOException;
import java.net.URL;

import fr.opensagres.xdocreport.core.debugger.AbstractRegistryDebugger;
import fr.opensagres.xdocreport.document.discovery.IXDocReportFactoryDiscovery;
import fr.opensagres.xdocreport.template.discovery.ITemplateEngineDiscovery;

/**
 * Abstract class for {@link IDebugger}.
 * 
 */
public abstract class AbstractDebugger extends AbstractRegistryDebugger implements IDebugger {

	// ------------------- Process report debug steps ---------------

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.opensagres.xdocreport.core.debug.IDebugger#startProcessReport()
	 */
	public void startProcessReport() {
		log("Start process report ");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.opensagres.xdocreport.core.debug.IDebugger#endProcessReport(long,
	 * java.lang.Throwable)
	 */
	public void endProcessReport(long time, Throwable e) {
		if (e == null) {
			log("End process report done with " + time + "(ms).");
		} else {
			log("End process report with error done with " + time + "(ms).", e);
		}
	}

	// ------------------- Preprocess debug steps ---------------

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * fr.opensagres.xdocreport.core.debug.IPreProcessorDebugger#startPreprocess
	 * (java.lang.String)
	 */
	public void startPreprocess(String entryName) {
		log("Start preprocess for the entry=" + entryName);
	}

	public void resultOfPreprocess(String entryName, String result) {
		log("Result preprocess for the entry=" + entryName + ": ");
		log(result);
	}

	public void endPreprocess(String entryName, long time, Throwable e) {
		if (e == null) {
			log("End preprocess for the entry=" + entryName + " done with "
					+ time + "(ms).");
		} else {
			log("End preprocess with error for the entry=" + entryName
					+ " done with " + time + "(ms).", e);
		}
	}

	// ------------------- Template engine debug steps ---------------

	/*
	 * (non-Javadoc)
	 * 
	 * @seefr.opensagres.xdocreport.core.debug.ITemplateEngineDebugger#
	 * startProcessTemplateEngine(java.lang.String, java.lang.String)
	 */
	public void startProcessTemplateEngine(String templateEngineId,
			String entryName, boolean useTemplateCache) {
		log("Start template engine id=" + templateEngineId + " for the entry="
				+ entryName + " with template cache=" + useTemplateCache);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seefr.opensagres.xdocreport.core.debug.ITemplateEngineDebugger#
	 * resultOfProcessTemplateEngine(java.lang.String, java.lang.String,
	 * java.lang.String)
	 */
	public void resultOfProcessTemplateEngine(String templateEngineId,
			String entryName, String result) {
		log("Result template engine id=" + templateEngineId
				+ "  for the entry=" + entryName + ": ");
		log(result);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seefr.opensagres.xdocreport.core.debug.ITemplateEngineDebugger#
	 * endProcessTemplateEngine(java.lang.String, java.lang.String, long,
	 * java.lang.Throwable)
	 */
	public void endProcessTemplateEngine(String templateEngineId,
			String entryName, long time, Throwable e) {
		if (e == null) {
			log("End template engine id=" + templateEngineId
					+ " for the entry=" + entryName + " done with " + time
					+ "(ms).");
		} else {
			log("End template engine id=" + templateEngineId
					+ " for the entry=" + entryName + " done with " + time
					+ "(ms).", e);
		}
	}

	// ------------------- Report loader debug steps ---------------
	/*
	 * (non-Javadoc)
	 * 
	 * @seefr.opensagres.xdocreport.core.debug.IXDocReportLoaderDebugger#
	 * startReportLoaderInitialisation()
	 */
	public void startReportLoaderInitialization() {
		log("Start report loader initialization.");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seefr.opensagres.xdocreport.core.debug.IXDocReportLoaderDebugger#
	 * endReportLoaderInitialization(long)
	 */
	public void endReportLoaderInitialization(long time) {
		log("End report loader initialization done with " + time + "(ms).");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seefr.opensagres.xdocreport.core.debug.IXDocReportLoaderDebugger#
	 * startLoadingPropertiesFile(java.net.URL)
	 */
	public void startLoadingPropertiesFile(URL url) {
		log("Start loading properties file=" + url.getPath());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seefr.opensagres.xdocreport.core.debug.IXDocReportLoaderDebugger#
	 * endLoadingPropertiesFile(java.net.URL, java.io.IOException)
	 */
	public void endLoadingPropertiesFile(URL url, IOException e) {
		if (e == null) {
			log("End loading properties file=" + url.getPath());
		} else {
			log(
					"End loading properties file=" + url.getPath()
							+ " with error.", e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seefr.opensagres.xdocreport.core.debug.IXDocReportLoaderDebugger#
	 * addReportFactoryForReportLoader(java.lang.String)
	 */
	public void addReportFactoryForReportLoader(String clazz) {
		log("Add report factory class=" + clazz);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seefr.opensagres.xdocreport.core.debug.IXDocReportLoaderDebugger#
	 * setDefaultTemplateEngineForReportLoader(java.lang.String,
	 * java.lang.String)
	 */
	public void setDefaultTemplateEngineForReportLoader(
			String templateEngineId, String clazz) {
		log("Set default template engine id=" + templateEngineId + ", class="
				+ clazz);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seefr.opensagres.xdocreport.core.debug.IXDocReportLoaderDebugger#
	 * ignoreTemplateEngineForReportLoader(java.lang.String, java.lang.String)
	 */
	public void ignoreTemplateEngineForReportLoader(String templateEngineId,
			String clazz) {
		log("Ignore default template engine id=" + templateEngineId
				+ ", class=" + clazz);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seefr.opensagres.xdocreport.core.debug.IXDocReportLoaderDebugger#
	 * dontImplementDiscovery(java.lang.String)
	 */
	public void dontImplementDiscovery(String clazz) {
		log("[" + clazz + "] don't implement discovery class "
				+ IXDocReportFactoryDiscovery.class.getName() + ", "
				+ ITemplateEngineDiscovery.class.getName());
	}

	public void cannotCreateClassForReportLoader(String clazz, Throwable e) {
		log("Cannot create an instance class=" + clazz, e);
	}

	/**
	 * Log information message.
	 * 
	 * @param message
	 */
	protected abstract void log(String message);

	/**
	 * Log error message with an exception.
	 * 
	 * @param message
	 * @param e
	 */
	protected abstract void log(String message, Throwable e);
}
