/**
 * Copyright (C) 2011 Angelo Zerr <angelo.zerr@gmail.com> and Pascal Leclercq <pascal.leclercq@gmail.com>
 *
 * All rights reserved.
 *
 * Permission is hereby granted, free  of charge, to any person obtaining
 * a  copy  of this  software  and  associated  documentation files  (the
 * "Software"), to  deal in  the Software without  restriction, including
 * without limitation  the rights to  use, copy, modify,  merge, publish,
 * distribute,  sublicense, and/or sell  copies of  the Software,  and to
 * permit persons to whom the Software  is furnished to do so, subject to
 * the following conditions:
 *
 * The  above  copyright  notice  and  this permission  notice  shall  be
 * included in all copies or substantial portions of the Software.
 *
 * THE  SOFTWARE IS  PROVIDED  "AS  IS", WITHOUT  WARRANTY  OF ANY  KIND,
 * EXPRESS OR  IMPLIED, INCLUDING  BUT NOT LIMITED  TO THE  WARRANTIES OF
 * MERCHANTABILITY,    FITNESS    FOR    A   PARTICULAR    PURPOSE    AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE,  ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package fr.opensagres.xdocreport.document.debug;

import fr.opensagres.xdocreport.core.debugger.IRegistryDebugger;
import fr.opensagres.xdocreport.document.IXDocReport;
import fr.opensagres.xdocreport.template.debug.ITemplateEngineDebugger;

/**
 * Debugger to track steps of process of report {@link IXDocReport} generation :
 * 
 * <ul>
 * <li>Track report loader which load default template engien to use and report
 * factory by using "xdocreport-discovery.properties".</li>
 * <li>Track start/end report generation.</li>
 * <li>Track start/result/end report preprocessing with
 * {@link IPreProcessorDebugger}.</li>
 * <li>Track start/result/end report merge Java model with document with
 * {@link ITemplateEngineDebugger}.</li>
 * </ul>
 * 
 */
public interface IDebugger extends IPreProcessorDebugger,
		ITemplateEngineDebugger, IRegistryDebugger {

	/**
	 * Start process of report generation.
	 */
	void startProcessReport();

	/**
	 * End process of report generation.
	 * 
	 * @param time
	 *            totally time for report generation.
	 * @param e
	 *            exception if there is error while report generation.
	 */
	void endProcessReport(long time, Throwable e);

}
